@@ -26,7 +26,6 @@ static int repeats;
 static char *winname;
 static char *command;
 static char *file;
-static char waitpid_done;
 
 static XWindowAttributes wattr;
 static Window victim;
@@ -62,7 +61,6 @@ static int gcd(int a, int b)
 
 static int set_aspect(int x, int y)
 {
-     D("set_aspect\n");
      int ox = xaspect;
      int oy = yaspect;
      int d = gcd(x, y);
@@ -138,6 +136,7 @@ static void adjust_window_size(void)
 static int place_window(void)
 {
      XWMHints *leader_change;
+     int i;
 
      if (!victim) return 0;
      if (!WINDOW) return 0;
@@ -147,23 +146,29 @@ static int place_window(void)
      XSync(display, False);
 
      D("Changing leader of window %x\n",victim);
-     if((leader_change = XGetWMHints(display, victim)))
+
+     if ((leader_change = XGetWMHints(display, victim)))
      {
 	  leader_change->flags = (leader_change->flags | WindowGroupHint);
 	  leader_change->window_group = wattr.root;
 	  XSetWMHints(display,victim,leader_change);
 	  XFree(leader_change);
      }
-      
-     D("Reparenting window %x into %x\n",victim,WINDOW);
-     XReparentWindow (display, victim, WINDOW, 0, 0);
-     XSync(display, False);
+     
+     D("Reparenting window %x into %x\n", victim, WINDOW);
+
+     /* Multiple XReparentWindow() are necessary on some X servers. */
+     for (i = 0; i < 30; i++)
+     {
+	  XReparentWindow (display, victim, WINDOW, 0, 0);
+	  XSync(display, False);
+     }
 
      /* XMapWindow() is necessary under OpenWindows. */
      XMapWindow(display, victim);
      XSync(display, False);
 
-     D("placewindow done\n");
+     D("place_window() done\n");
 
      return 1;
 }
@@ -198,8 +203,9 @@ static int my_strcmp(char *windowname, char *name)
 	  return strcasecmp(name+1, windowname);
      case '*':
 	  return strncasecmp(name+1, windowname, strlen(name)-1);
+     default:
+	  return strncasecmp(name, windowname, strlen(name));
      }
-     return strcasecmp(name, windowname);
 }
 
 static char check_window_name(Window w, char *name)
@@ -315,7 +321,7 @@ static void run_app(char **argv)
      }
      
      execvp(argv[0], argv);
-     D("Execvp failed..%d\n", errno);
+     D("Execvp failed. (errno=%d)\n", errno);
      exit(EX_UNAVAILABLE);
 }
 
@@ -443,13 +449,13 @@ static void check_pipe_fd_events(void)
      {
 	  if (errno == EINTR) return;
 	  D("Winddata read error, exiting\n");
-	  _exit(EX_UNAVAILABLE);
+	  exit(EX_UNAVAILABLE);
      }
   
      if (n == 0)
      {
 	  D("Winddata EOF, exiting\n");
-	  _exit(EX_UNAVAILABLE);
+	  exit(EX_UNAVAILABLE);
      }
   
      if (n != sizeof(windata))
@@ -474,65 +480,73 @@ static void check_pipe_fd_events(void)
      }
 }
 
-static void handle_app(int pid)
+static void check_all_events()
 {
-     int status = 0;
      struct timeval tv;
      int maxfd;
      fd_set fds;
 
+     FD_ZERO(&fds);
+     FD_SET(ConnectionNumber(display), &fds);
+     FD_SET(pipe_fd, &fds);
+
+     maxfd = MAX(ConnectionNumber(display), pipe_fd);
+     
+     tv.tv_sec = SELECT_TIMEOUT_SEC;
+     tv.tv_usec = SELECT_TIMEOUT_USEC;
+     
+     D("SELECT IN\n");
+     if (select(maxfd + 1, &fds, NULL, NULL, &tv) > 0)
+     {
+	  D("SELECT OUT\n");
+	  if (FD_ISSET(pipe_fd, &fds))
+	       check_pipe_fd_events();
+     }
+     else
+     {
+	  D("SELECT ERROR\n");
+     }
+
+     if (display)
+     {
+	  check_x_events();
+     }
+}
+
+static void handle_app(int pid)
+{
+     int status;
+
      if (flags & H_SWALLOW)
      {
-	  while (!waitpid_done)
+	  while (!waitpid(pid, &status, WNOHANG))
 	  {
-	       FD_ZERO(&fds);
-	       FD_SET(ConnectionNumber(display), &fds);
-	       FD_SET(pipe_fd, &fds);
-
-	       maxfd = MAX(ConnectionNumber(display), pipe_fd);
-
-	       tv.tv_sec = SELECT_TIMEOUT_SEC;
-	       tv.tv_usec = SELECT_TIMEOUT_USEC;
-
-	       D("SELECT IN\n");
-	       if (select(maxfd + 1, &fds, NULL, NULL, &tv) > 0)
-	       {
-		    D("SELECT OUT\n");
-		    if (FD_ISSET(pipe_fd, &fds))
-			 check_pipe_fd_events();
-	       }
-	       else
-	       {
-		    D("SELECT ERROR\n");
-	       }
-
-	       if (display)
-		    check_x_events();
+	       check_all_events();
 	  }
      }
+     else
+     {
+	  waitpid(pid, &status, 0);
+     }
 
      if (flags & H_DAEMON)
+     {
 	  exit(0);
-     
+     }
+
      if (!WIFEXITED(status))
      {
 	  D("Process dumped core or something...\n");
 	  exit(EX_UNAVAILABLE);
      }
+
      if (WEXITSTATUS(status) && !(flags & H_IGNORE_ERRORS))
      {
 	  D("Process exited with error code: %d\n", WEXITSTATUS(status));
 	  exit(WEXITSTATUS(status));
      }
-     D("exited ok!\n");
-}
 
-static void sigchild(int signum)
-{
-     int status;
-     D("SIGCHILD\n");
-     waitpid_done = 1;
-     while (waitpid(-1, &status, WNOHANG) > 0);
+     D("Exited OK!\n");
 }
 
 int main(int argc, char **argv)
@@ -567,8 +581,6 @@ int main(int argc, char **argv)
        file,
        command);
 
-     signal(SIGCHLD, sigchild);
-
      while (repeats > 0)
      {
 	  char *argv[10];
@@ -601,12 +613,10 @@ int main(int argc, char **argv)
 	  /* Has to be done before forking */
 	  if (setup_swallow())
 	  {
-	       D("setup_swallow() failed!!!!\n");
+	       D("setup_swallow() failed\n");
 	       flags &=~ H_SWALLOW;
 	  }
 
-	  D("Running %s (repeats=%d loops=%d)\n",command,repeats,loops);
-
 	  if ((pid = fork()) == -1)
 	       exit(EX_UNAVAILABLE);
       
@@ -614,15 +624,16 @@ int main(int argc, char **argv)
 	  {
 	       XSelectInput(display, WINDOW, 0);
 	       XSelectInput(display, wattr.root, 0);
+	       D("Running %s\n", command);
 	       run_app(argv);
 	  } else {
-	       D("waiting for (%d)\n", pid);
+	       D("Waiting for pid=%d\n", pid);
 	       handle_app(pid);
-	       D("wait done repeats=%d loops=%d\n", repeats, loops);
+	       D("Wait done (repeats=%d, loops=%d)\n", repeats, loops);
 	       if (repeats < MAXINT)
 		    repeats -= loops;
 	  }
      }
-     _exit(0);
-}
 
+     exit(0);
+}
