@@ -1099,7 +1099,7 @@ NPError NPP_NewStream(NPP instance,
      
      if (!find_command(instance, 1) && !find_command(instance, 0))
      {
-	  NPN_Status(instance, "MozPlugger: No appropriate application found !");
+	  NPN_Status(instance, "MozPlugger: No appropriate application found.");
 	  return NPERR_GENERIC_ERROR;
      }
 
