@@ -16,34 +16,20 @@
 #include <X11/Xatom.h>
 #include <sys/socket.h>
 
-#define DEBUG_FILENAME "/tmp/mozdebug"
-
 #ifndef MAXINT
 #define MAXINT 0x7fffffff
 #endif
 
-/*****************************************************************************
- * Active wait in useconds used with the flag swallow
- *****************************************************************************/
-#define SCAN_WAIT_TIME 500
+#define MAX(X,Y) ((X)>(Y)?(X):(Y))
 
 /*****************************************************************************
- * Passive wait in useconds used with the flag event_swallow
+ * Timeout for select()
  *****************************************************************************/
-#define EVENT_WAIT_TIME (500 * 1000)
+#define SELECT_TIMEOUT_SEC  0
+#define SELECT_TIMEOUT_USEC 100*1000
 
 /*****************************************************************************
- * Maximum length of the swallow stage in seconds
- *****************************************************************************/
-#define SWALLOW_MAX_LEN 60
-
-/*****************************************************************************
- * How many times to reparent the window when found
- *****************************************************************************/
-#define REPARENT_REPEATS 25
-
-/*****************************************************************************
- * Options
+ * Flags
  *****************************************************************************/
 #define H_LOOP 		0x1
 #define H_DAEMON 	0x2
@@ -56,7 +42,7 @@
 #define H_SWALLOW 	0x100
 #define H_MAXASPECT 	0x200
 #define H_FILL 		0x400
-#define H_EVENT_SWALLOW 0x800
+#define H_HIDDEN        0x800
 #define H_CONTROLS 	0x1000
 
 /*****************************************************************************
@@ -67,5 +53,3 @@ void close_debug();
 #endif
 void D(char *fmt, ...);
 void my_kill(int pid);
-void max_aspect_window(Display *display, int victim, int w, int h);
-void fill_window(Display *display, int victim, int w, int h);
